﻿using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;
using System.Diagnostics;
using System.Linq;

namespace metastrings
{
    public class define : IDisposable
    {
        public define(string serverUrl)
        {
            m_serverUrl = serverUrl;
        }

        public void Dispose()
        {
            if (m_streamWriter != null)
            {
                try { m_streamWriter.Dispose(); } catch { }
                m_streamWriter = null;
            }
            ScopeTiming.RecordScope("define.Total", m_totalTimer);
        }

        public async Task BeginAsync(ListDictionary<string, string> headers, IEnumerable<string> columns)
        {
            try
            {
                var timer = ScopeTiming.StartTiming();
                m_request = WebRequest.CreateHttp($"{m_serverUrl}?type=definecsv");
                m_request.Method = "POST";

                var requestStream = await m_request.GetRequestStreamAsync().ConfigureAwait(false);
                m_streamWriter = new StreamWriter(requestStream, Encoding.UTF8);
                ScopeTiming.RecordScope("define.Open", timer);

                StringBuilder sb = new StringBuilder();
                sb.AppendLine(CsvPrep.CreateCsv(headers.Keys));
                sb.AppendLine(CsvPrep.CreateCsv(headers.Values));
                sb.Append("value,");
                sb.AppendLine(CsvPrep.CreateCsv(columns));
                string header = sb.ToString();
                await m_streamWriter.WriteAsync(header).ConfigureAwait(false);
                ScopeTiming.RecordScope("define.Headers", timer);
            }
            catch (Exception exp)
            {
                throw Utils.HandleException(exp);
            }
        }

        public async Task WriteRecordAsync(IEnumerable<object> values)
        {
            int colCount = values.Count();
            await m_streamWriter.WriteLineAsync(CsvPrep.CreateCsv(values)).ConfigureAwait(false);
        }

        public async Task EndAsync()
        {
            try
            {
                var timer = ScopeTiming.StartTiming();
                m_streamWriter.Close();
                m_streamWriter.Dispose();
                m_streamWriter = null;
                ScopeTiming.RecordScope("define.Close", timer);

                using (var response = await m_request.GetResponseAsync().ConfigureAwait(false))
                using (var responseStream = response.GetResponseStream())
                { }
                ScopeTiming.RecordScope("define.End", timer);
            }
            catch (Exception exp)
            {
                throw Utils.HandleException(exp);
            }
        }

        private Stopwatch m_totalTimer = ScopeTiming.StartTiming();
        private HttpWebRequest m_request;
        private StreamWriter m_streamWriter;
        private string m_serverUrl;
    }
}
