﻿using System;
using System.Collections.Generic;

namespace metastrings
{
    public class Define
    {
        public string type { get; set; }

        // item value -> metadata name-> value
        public Dictionary<object, Dictionary<string, object>> metadata { get; set; }

        // helper function
        public void SetData(object itemValue, string metadataName, object metadataValue)
        {
            if (metadata == null)
                metadata = new Dictionary<object, Dictionary<string, object>>();

            Dictionary<string, object> metadataDict;
            if (!metadata.TryGetValue(itemValue, out metadataDict))
            {
                metadataDict = new Dictionary<string, object>();
                metadata.Add(itemValue, metadataDict);
            }

            metadataDict[metadataName] = metadataValue;
        }
    }

    public class GetRequest
    {
        public string type { get; set; }
        public List<object> values { get; set; }
    }

    public class Criteria // WHERE
    {
        public string name { get; set; }
        public string op { get; set; }
        public string paramName { get; set; }
    }

    public enum CriteriaCombine { AND, OR }
    public class CriteriaSet
    {
        public CriteriaCombine combine { get; set; } = CriteriaCombine.AND;
        public List<Criteria> criteria { get; set; } = new List<Criteria>();

        public CriteriaSet() { }

        public CriteriaSet(Criteria criteria)
        {
            AddCriteria(criteria);
        }

        public static List<CriteriaSet> GenWhere(Criteria criteria)
        {
            return new List<CriteriaSet>{ new CriteriaSet(criteria) };
        }

        public static IEnumerable<CriteriaSet> GenWhere(IEnumerable<Criteria> criteria)
        {
            CriteriaSet set = new CriteriaSet();
            foreach (var c in criteria)
                set.AddCriteria(c);
            return new[] { new CriteriaSet() };
        }

        public void AddCriteria(Criteria criteria)
        {
            this.criteria.Add(criteria);
        }
    }

    public class Order // ORDER BY
    {
        public string field { get; set; }
        public bool descending { get; set; }
    }

    public class QueryGetRequest
    {
        public string type { get; set; } // FROM
        public List<CriteriaSet> where { get; set; }
        public List<Order> orderBy { get; set; }
        public int limit { get; set; }

        public Dictionary<string, object> cmdParams { get; set; }

        public bool prefixesOnly { get; set; }
    }

    public class Select : QueryGetRequest
    {
        public List<string> select { get; set; }
    }

    public class GetResponse
    {
        public List<Dictionary<string, object>> metadata { get; set; }
    }

    public class Delete
    {
        public string type { get; set; }
        public List<object> values { get; set; }
    }

    public class Drop
    {
        public string type { get; set; }
    }

    public class Reset
    {
        public bool includeNameValues { get; set; }
    }

    public class Timing
    {
        public bool reset { get; set; }
    }
}
