﻿using System;
using System.Threading.Tasks;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using System.Globalization;

namespace metastrings
{
    public class query : IDisposable
    {
        public query(string serverUrl)
        {
            m_serverUrl = serverUrl;
        }

        public void Dispose()
        {
            if (m_csvParser != null)
            {
                try { m_csvParser.Dispose(); } catch { }
                m_csvParser = null;
            }

            if (m_response != null)
            {
                try { m_response.Dispose(); } catch { }
                m_response = null;
            }
        }

        public async Task ExecuteAsync(Select select)
        {
            try
            {
                var req = WebRequest.CreateHttp($"{m_serverUrl}?type=query");
                req.Method = "POST";

                var timer = ScopeTiming.StartTiming();
                using (var requestStream = await req.GetRequestStreamAsync().ConfigureAwait(false))
                using (var sw = new StreamWriter(requestStream, Encoding.UTF8))
                    await sw.WriteAsync(Utils.Serialize(select)).ConfigureAwait(false);
                ScopeTiming.RecordScope("query.exec.request", timer);

                m_response = await req.GetResponseAsync().ConfigureAwait(false);
                m_csvParser = new CsvParser(new StreamReader(m_response.GetResponseStream(), Encoding.UTF8));
                ScopeTiming.RecordScope("query.exec,response", timer);

                if (!await ReadAsync().ConfigureAwait(false))
                    throw new MetaStringsException("Column header row not read");
                for (int idx = 0; idx < FieldCount; ++idx)
                    m_dictNameIdxMap[this[idx]] = idx;
            }
            catch (Exception exp)
            {
                throw Utils.HandleException(exp);
            }
        }

        public async Task<bool> ReadAsync()
        {
            return await m_csvParser.ReadAsync().ConfigureAwait(false);
        }

        public int FieldCount => m_csvParser.Count;
        public string this[int i] => m_csvParser[i];
        public string this[string col] => m_csvParser[GetColumnIdx(col)];

        public int GetColumnIdx(string col)
        {
            int idx;
            if (!m_dictNameIdxMap.TryGetValue(col, out idx))
                throw new MetaStringsException($"Column not found: {col}");
            return idx;
        }

        private string m_serverUrl;

        private WebResponse m_response;
        private CsvParser m_csvParser;

        private Dictionary<string, int> m_dictNameIdxMap = new Dictionary<string, int>();
    }
}
