﻿using System;

using MySql.Data.MySqlClient;

namespace metastrings
{
    public class Connection : IDisposable
    {
        public Connection(string connStr)
        {
            MyDb = new MySqlConnection(connStr);
            MyDb.Open();
        }

        public void Dispose()
        {
            if (MyTrans != null)
            {
                MyTrans.Dispose();
                MyTrans = null;
            }

            if (MyDb != null)
            {
                MyDb.Dispose();
                MyDb = null;
            }
        }

        public void BeginTrans()
        {
            ++m_transCount;

            if (m_transCount == 1)
                MyTrans = MyDb.BeginTransaction();
        }

        public void Commit()
        {
            --m_transCount;

            if (m_transCount < 0)
                throw new Exception("Too many Commits");

            if (m_transCount == 0)
            {
                MyTrans.Commit();
                MyTrans.Dispose();
                MyTrans = null;
            }
        }

        public void Rollback()
        {
            --m_transCount;

            if (m_transCount < 0)
                throw new Exception("Too many Rollbacks");

            if (m_transCount == 0)
            {
                MyTrans.Rollback();
                MyTrans.Dispose();
                MyTrans = null;
            }
        }

        public MySqlConnection MyDb;
        public MySqlTransaction MyTrans;

        private int m_transCount = 0;
    }
}
