﻿using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Configuration;

using MySql.Data.MySqlClient;

namespace metastrings
{
    public class Context : IDisposable
    {
        public Context(string connStrCfgName = "metastrings")
        {
            var connStrObj = ConfigurationManager.ConnectionStrings[connStrCfgName];
            if (connStrObj == null)
                throw new ConfigurationErrorsException("metastrings connection string missing from config");

            var connStr = connStrObj.ConnectionString;
            if (string.IsNullOrWhiteSpace(connStr))
                throw new ConfigurationErrorsException("metastrings connection string empty in config");

            DbConn = new Connection(connStr);
        }

        public void Dispose()
        {
            if (m_postItemOps.Count > 0)
                throw new MetaStringsException("Post ops remain; call ProcessPostOpsAsync before disposing the metastrings context");

            DbConn.Dispose();
            DbConn = null;
        }

        public Connection DbConn;

        public async Task<int> ProcessPostOpsAsync()
        {
            int affected = 0;

            if (m_postItemOps.Count == 0)
                return affected;

            var totalTimer = ScopeTiming.StartTiming();

            try
            {
                var batches = Utils.BatchUp(m_postItemOps, 32 * 1024);
                m_postItemOps.Clear();

                DbConn.BeginTrans();
                foreach (string stmt in batches)
                {
                    try
                    {
                        using (var cmd = new MySqlCommand(stmt, DbConn.MyDb, DbConn.MyTrans))
                            affected += await cmd.ExecuteNonQueryAsync().ConfigureAwait(false);
                    }
                    catch (Exception exp)
                    {
                        throw new MetaStringsException("ProcessPostOpsAsync EXCEPTION", exp);
                    }
                }
                DbConn.Commit();

                return affected;
            }
            finally
            {
                ScopeTiming.RecordScope("ProcessItemPostOps", totalTimer);
            }
        }
        public void AddPostItemOp(string sql)
        {
            m_postItemOps.Add($"{sql};\r\n");
        }

        public void ClearPostItemOps()
        {
            m_postItemOps.Clear();
        }
        private List<string> m_postItemOps = new List<string>();
    }
}
