﻿using System;
using System.Threading.Tasks;
using System.Collections.Generic;
using System.Data;

using MySql.Data.MySqlClient;

namespace metastrings
{
    public static class Items
    {
        public static void Reset(Context ctxt)
        {
            using (var cmd = new MySqlCommand("DELETE FROM items", ctxt.DbConn.MyDb, ctxt.DbConn.MyTrans))
                cmd.ExecuteNonQuery();
        }

        public static async Task<long> GetIdAsync(Context ctxt, short typeId, long valueId, bool noCreate = false)
        {
            var totalTimer = ScopeTiming.StartTiming();
            try
            {
                Exception lastExp = null;
                for (int tryCount = 1; tryCount <= 4; ++tryCount)
                {
                    try
                    {
                        using (var cmd = new MySqlCommand("GetItemId", ctxt.DbConn.MyDb, ctxt.DbConn.MyTrans))
                        {
                            cmd.CommandType = CommandType.StoredProcedure;
                            cmd.Parameters.AddWithValue("typeIdParam", typeId);
                            cmd.Parameters.AddWithValue("valueIdParam", valueId);
                            cmd.Parameters.AddWithValue("noCreate", noCreate);
                            object idObj = await cmd.ExecuteScalarAsync().ConfigureAwait(false);
                            long id = Convert.ToInt64(idObj);
                            return id;
                        }
                    }
                    catch (Exception exp)
                    {
                        lastExp = exp;
                    }
                }

                throw new MetaStringsException("Items.GetId failed after a few tries", lastExp);
            }
            finally
            {
                ScopeTiming.RecordScope("Items.GetId", totalTimer);
            }
        }

        public static async Task<Dictionary<short, long>> GetItemDataAsync(Context ctxt, long itemId)
        {
            var retVal = new Dictionary<short, long>();
            string sql = $"SELECT nameid, valueid FROM itemnamevalues WHERE itemid = {itemId}";
            using (var cmd = new MySqlCommand(sql, ctxt.DbConn.MyDb, ctxt.DbConn.MyTrans))
            using (var reader = await cmd.ExecuteReaderAsync().ConfigureAwait(false))
            {
                while (await reader.ReadAsync().ConfigureAwait(false))
                    retVal[reader.GetInt16(0)] = reader.GetInt64(1);
            }
            return retVal;
        }

        public static void SetItemData(Context ctxt, long itemId, Dictionary<short, long> itemData)
        {
            foreach (var kvp in itemData)
            {
                string sql;
                if (kvp.Value >= 0) // add-or-update it
                {
                    sql =
                        "INSERT INTO itemnamevalues (itemid, nameid, valueid) " +
                        $"VALUES ({itemId}, {kvp.Key}, {kvp.Value})" +
                        $"ON DUPLICATE KEY UPDATE valueid = {kvp.Value}";
                }
                else // remove it
                    sql = $"DELETE FROM itemnamevalues WHERE itemid = {itemId} AND nameid = {kvp.Key}";
                ctxt.AddPostItemOp(sql);
            }
        }

        public static async Task DeleteAsync(Context ctxt, long itemId)
        {
            string sql = $"DELETE FROM items WHERE id = {itemId}";
            using (var cmd = new MySqlCommand(sql, ctxt.DbConn.MyDb, ctxt.DbConn.MyTrans))
                await cmd.ExecuteNonQueryAsync().ConfigureAwait(false);
        }
    }
}
