﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Diagnostics;
using System.Linq;

namespace metastrings
{
    public static class MyFileInfoMgr
    {
        public static Dictionary<string, MyFileInfo> GetFileInfo(IEnumerable<string> dirPaths)
        {
            var fileInfo = new Dictionary<string, MyFileInfo>();
            foreach (string dirPath in dirPaths)
            {
                Console.WriteLine("Processing Directory: {0}", dirPath);

                Console.Write("Getting file paths... ");
                var filePaths = Directory.GetFiles(dirPath, "*", SearchOption.AllDirectories);
                Console.WriteLine("Files: {0}", filePaths.Length);

                Console.Write("Getting file info...");
                GetFilesInfo(filePaths, fileInfo);
                Console.WriteLine(" Done!");
            }
            return fileInfo;
        }

        private static void GetFilesInfo(IEnumerable<string> files, Dictionary<string, MyFileInfo> retVal)
        {
            foreach (var filePath in files)
            {
                MyFileInfo newVal = new MyFileInfo(filePath);

                FileInfo fi = new FileInfo(filePath);
                newVal.AddMetadata("length#", fi.Length); // # is for CSV
                newVal.AddMetadata("created", fi.CreationTimeUtc);
                newVal.AddMetadata("lastModified", fi.LastWriteTimeUtc);
                newVal.AddMetadata("ext", fi.Extension.Trim('.').ToLower());
                newVal.AddMetadata("readOnly", fi.IsReadOnly);

                FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(filePath);
                newVal.AddMetadata("name", fvi.FileName);
                newVal.AddMetadata("company", fvi.CompanyName);
                newVal.AddMetadata("product", fvi.ProductName);
                newVal.AddMetadata("description", fvi.FileDescription);
                newVal.AddMetadata("version", fvi.FileVersion);
                newVal.AddMetadata("language", fvi.Language);
                newVal.AddMetadata("copyright", fvi.LegalCopyright);

                retVal[newVal.FilePath] = newVal;
            }
        }
    }
}
