﻿using System;
using System.Threading;
using System.Collections.Generic;

using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace metastrings
{
    [TestClass]
    public class DefineTest
    {
        [TestMethod]
        public void TestDefine()
        {
            using (var ctxt = new Context())
            {
                Items.Reset(ctxt);

                var define = new Define();

                define.type = "fun";

                define.SetData("some", "num", 42);
                define.SetData("some", "str", "foobar");

                using (Command cmd = new Command(ctxt))
                    cmd.DefineAsync(define).Wait();
                ctxt.ProcessPostOpsAsync().Wait();

                long itemId = Items.GetIdAsync(ctxt, Types.GetIdAsync(ctxt, define.type).Result, Values.GetIdAsync(ctxt, "some").Result).Result;
                var itemData = NameValues.GetMetadataValuesAsync(ctxt, Items.GetItemDataAsync(ctxt, itemId).Result).Result;

                Assert.AreEqual(42.0, itemData["num"]);
                Assert.AreEqual("foobar", itemData["str"]);

                {
                    Select select = new Select()
                    {
                        type = define.type,
                        select = new List<string>{ "value", "str" },
                        where = CriteriaSet.GenWhere(new Criteria() { op = "matches", paramName = "@search", name = "str" })
                    };
                    select.cmdParams = new Dictionary<string, object>();
                    select.cmdParams["@search"] = "foobar";
                    using (Command cmd = new Command(ctxt))
                    {
                        using (var reader = cmd.GetReaderAsync(select).Result)
                        {
                            if (!reader.ReadAsync().Result)
                                Assert.Fail();

                            var val = reader.GetString(0);
                            var str = reader.GetString(1);

                            Assert.AreEqual("some", val);
                            Assert.AreEqual("foobar", str);

                            if (reader.ReadAsync().Result)
                                Assert.Fail();
                        }
                    }
                }

                define.SetData("some", "num", 43.0);
                define.SetData("some", "str", null); // remove the metadata

                using (Command cmd = new Command(ctxt))
                    cmd.DefineAsync(define).Wait();
                ctxt.ProcessPostOpsAsync().Wait();

                itemData = NameValues.GetMetadataValuesAsync(ctxt, Items.GetItemDataAsync(ctxt, itemId).Result).Result;
                Assert.IsTrue(!itemData.ContainsKey("str"));
                Assert.AreEqual(43.0, itemData["num"]);

                using (Command cmd = new Command(ctxt))
                {
                    cmd.DeleteAsync(new Delete() { type = define.type, values = new List<object> { "some" } }).Wait();
                    int affected = ctxt.ProcessPostOpsAsync().Result;
                    Assert.AreEqual(1, affected);
                }

                {
                    Select select = new Select()
                    {
                        type = define.type,
                        select = new List<string> { "value" }
                    };
                    using (Command cmd = new Command(ctxt))
                    using (var reader = cmd.GetReaderAsync(select).Result)
                    {
                        if (reader.ReadAsync().Result) // should be gone
                            Assert.Fail();
                    }
                }

                Items.Reset(ctxt);
            }
        }
    }
}
