﻿using System;
using System.Collections.Generic;

using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace metastrings
{
    [TestClass]
    public class ItemTests
    {
        [TestMethod]
        public void TestEnsureItem()
        {
            using (var ctxt = new Context())
            {
                Items.Reset(ctxt);

                short typeId = Types.GetIdAsync(ctxt, "foo").Result;
                long valueId = Values.GetIdAsync(ctxt, "bar").Result;

                long id0 = Items.GetIdAsync(ctxt, typeId, valueId).Result;
                long id1 = Items.GetIdAsync(ctxt, typeId, valueId).Result;

                Assert.AreEqual(id0, id1);

                Items.Reset(ctxt);
            }
        }

        [TestMethod]
        public void TestItemData()
        {
            using (var ctxt = new Context())
            {
                Items.Reset(ctxt);

                short typeId = Types.GetIdAsync(ctxt, "blet").Result;
                long itemId = Items.GetIdAsync(ctxt, typeId, Values.GetIdAsync(ctxt, "monkey").Result).Result;

                var itemData = new Dictionary<short, long>();
                itemData[Names.GetIdAsync(ctxt, typeId, "foo").Result] = Values.GetIdAsync(ctxt, "bar").Result;
                itemData[Names.GetIdAsync(ctxt, typeId, "something").Result] = Values.GetIdAsync(ctxt, "else").Result;
                Items.SetItemData(ctxt, itemId, itemData);
                ctxt.ProcessPostOpsAsync().Wait();

                var metadata = Items.GetItemDataAsync(ctxt, itemId).Result;

                foreach (var kvp in metadata)
                {
                    string nameVal = Names.GetNameAsync(ctxt, kvp.Key).Result.name;
                    string valueVal = Values.GetValueAsync(ctxt, kvp.Value).Result.ToString();
                    if (nameVal == "foo")
                        Assert.AreEqual("bar", valueVal);
                    else if (nameVal == "something")
                        Assert.AreEqual("else", valueVal);
                    else
                        Assert.Fail("Name not recognized: " + nameVal);
                }

                Items.Reset(ctxt);
            }
        }
    }
}
