﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace metastrings
{
    [TestClass]
    public class ItemNameValueTests
    {
        [TestMethod]
        public void TestItemNameValues()
        {
            using (var ctxt = new Context())
            {
                NameValues.Reset(ctxt);

                short typeId = Types.GetIdAsync(ctxt, "ape").Result;
                short bletNameId = Names.GetIdAsync(ctxt, typeId, "blet").Result;
                long monkeyValueId = Values.GetIdAsync(ctxt, "monkey").Result;

                long itemId0 = Items.GetIdAsync(ctxt, typeId, monkeyValueId).Result;
                long itemId1 = Items.GetIdAsync(ctxt, typeId, monkeyValueId).Result;
                Assert.AreEqual(itemId0, itemId1);

                Items.DeleteAsync(ctxt, itemId1).ConfigureAwait(false);
                long itemId2 = Items.GetIdAsync(ctxt, typeId, monkeyValueId).Result;
                Assert.AreNotEqual(itemId0, itemId2);

                var metaDict = new Dictionary<short, long>();
                short fredNameId = Names.GetIdAsync(ctxt, typeId, "fred").Result;
                long earnyValueId = Values.GetIdAsync(ctxt, "earny").Result;
                metaDict[fredNameId] = earnyValueId;
                Items.SetItemData(ctxt, itemId2, metaDict);
                ctxt.ProcessPostOpsAsync().Wait();

                var outMetaDict = Items.GetItemDataAsync(ctxt, itemId2).Result;
                Assert.AreEqual(1, outMetaDict.Count);
                Assert.IsTrue(outMetaDict.ContainsKey(fredNameId));
                Assert.AreEqual(earnyValueId, outMetaDict[fredNameId]);

                NameValues.Reset(ctxt);
            }
        }
    }
}
